#include "RSSData.hpp"
#include "RssReader.hpp"
//
// If the title and date of the RSS items are the same, they are considered the same.
//
Bool operator==(RSSItemConstRef item1, RSSItemConstRef item2)
{
    return item1.title == item2.title && item1.date == item2.date;
}

//
// Constructor definition.
//
RSSItem::RSSItem(SFXAnsiStringConstRef rsstitle,
SFXAnsiStringConstRef rssdescription,
SFXDateConstRef rssdate,
Bool rssalreadyRead)

: title(rsstitle), description(rssdescription),
date(rssdate), alreadyRead(rssalreadyRead) {
    return;
}

//
// Constructor definition.
//
RSSFeed::RSSFeed(SFXAnsiStringConstRef title,
        SFXAnsiStringConstRef url, SFXDateConstRef date)
    : _title(title), _url(url), _date(date)
{
    return;
}

//
// Destructor definition.
//
RSSFeed::~RSSFeed(Void)
{
    RSSItemPtr item;
    // An iterator is obtained and all data is released
    SFXList<RSSItemPtr>::Iterator itor(_itemList.GetFirstIterator());
    while(itor.HasNext()) {
        item = itor.GetNext();
        ::delete item;
    }
    return;
}

//
// RSS analyze.
//
SFCError RSSFeed::Parse(SFXAnsiStringConstRef xml)
{
    SFCError error;
    SFXXMLDOMParser parser;
    SFXXMLDocumentPtr root;
    SFXXMLNodePtr child;
    SFXAnsiString title;
    SFXAnsiString description;
    SFXDate date;
    RSSItemPtr item;
    SInt32 listcount;
    SInt32 count = 0;
    SFXAnsiString temp;
    // Analyze XML document
    error = parser.Parse(xml);
	// If parse failed
    if (error != SFERR_NO_ERROR) return SFERR_FAILED;
	// Get the root
	root = parser.GetDocument(); 
    if (root == null) return SFERR_FAILED;
	// Search for the title tag under the channel tag
    SFXList<SFXXMLNodePtr>* list = root->GetElementsByTagName("channel");
    if (list != null && list->GetSize() > 0) 
    {
        child = list->Get(0); 
        for (child = child->GetFirstChild() ; child != null && error == SFERR_NO_ERROR ; child = child->GetNextSibling()) 
        {
			// If it is the title tag
            if (SFXAnsiString(child->GetNodeName()) == "title") 
            {
                temp = child->GetText();
                if (_title.IsEmptyCString()) 
                {
                    error = SFXTextEncoding::UTF8ToShiftJIS(temp, &_title);
                }
            }

 			else if (SFXAnsiString(child->GetNodeName()) == "dc:date") 
            {
                        date.Parse("YYYY-MM-DD%Thh:mm:ss+",
                        child->GetText());
            }
        }

		// For each item tag
        SFXList<SFXXMLNodePtr>* list = root->GetElementsByTagName("item");
        if (list != null && list->GetSize() > 0) 
        {
            for (listcount=0; listcount < (list->GetSize()); listcount++)  
            {
                child = list->Get(listcount); 
                for (child = child->GetFirstChild() ; child != null && error == SFERR_NO_ERROR ; child = child->GetNextSibling()) 
                {
                    // If it is a title tag
                    if (SFXAnsiString(child->GetNodeName()) == "title") 
                    {
                        // Get the text of the node
                        temp = child->GetText();
                        // UTF-8 is converted into Shift_JIS
                        error = SFXTextEncoding::UTF8ToShiftJIS(temp, &title);
                    } 
                    // If it is a description tag
                    else if (SFXAnsiString(child->GetNodeName()) == "description") 
                    {
						// Get the text of the node
                        temp = child->GetText();
						// UTF-8 is converted into Shift_JIS
                        error = SFXTextEncoding::UTF8ToShiftJIS(temp, &description);
                    }

					// // If it is a dc: date tag
                    else if (SFXAnsiString(child->GetNodeName()) == "dc: date") 
                    {
                        // Convert string into date
                        date.Parse("YYYY-MM-DD%Thh:mm:ss+",
                        child->GetText());
                    }
                }
                item = ::new RSSItem(title, description, date);
                if (item != null) 
                {
                    // If not duplicated
                    if (!Find(*item)) 
                    {
                        // Insert item
                        error = _itemList.Insert(count, item);
                        if (error == SFERR_NO_ERROR) 
                        {
                            ++count;
                        }
                        else 
                        {
                            ::delete item;
                        }
                    }
                    else 
                    {
                        ::delete item;
                    }
                }
                else 
                {
                    error = SFERR_NO_MEMORY;
                }
            }
        }
    }
    else 
    {
        TRACE ("list empty");
    }
    // Error
    if (error != SFERR_NO_ERROR) 
    {
        _title = "Failed";
    }
    SFXHelper::dbgprintf("[SGXAWSParser] ParseXML END %d", error);
    if (error == SFERR_NO_ERROR) 
    {
        // Save to file
        error = RSSFeed::SaveToFile();
    }
    return error;
}
//
// File saving.
// The data specification is the following:
// - each line displays one RSS item,
// - each RSS item is delimited with DELIMITER_STR,
// - when DELIMITER_STR is <>, a line becomes
// title<>description<>topic<>date<>alreadyRead\r\n
//
SFCError RSSFeed::SaveToFile(Void)
{
    SFCError error;
    SFXPath temp;
    SFXFile file;
    SFXAnsiStringStreamWriter writer;
    RSSItemPtr item;
    SFXPath filename(DIRECTORY_NAME + _filename);

    // Get path for temporary saving
    error = SFXFile::GetTemporaryPath(DIRECTORY_PATH, &temp);

    if (error == SFERR_NO_ERROR) {
		bool exists;
        error =SFXFile::Exists(	temp ,&exists);
        if (exists==false)
        {
            file.Create(temp,true);
        }
        // Open file
        error = file.OpenReadWrite(temp);
        if (error == SFERR_NO_ERROR) {
            // Get the writing stream
            error = file.GetStreamWriter(4096, &writer);
            if (error == SFERR_NO_ERROR) {
                SFXList<RSSItemPtr>::Iterator itor(_itemList.GetFirstIterator());
                // All data is written in file
                while (itor.HasNext()) {
                    // Get iterator
                    item = itor.GetNext();
                    // Write RSSItem onto file
                    writer << item->title << DELIMITER_STR 
					<< item->description  << DELIMITER_STR
                    << SFXAnsiString::Format("%u", item->date.AsUInt32()) << DELIMITER_STR
					<< ((item->alreadyRead) ? "1" : "0")
                    << "\r\n";
                    // If Flush is not called, data is not actually written in the file
                    writer.Flush();
                }
            }
            file.Close();
        }
    }
    if (error == SFERR_NO_ERROR) {
        // Delete file
        SFXFile::Remove(filename);
        // Rename file
        error = SFXFile::Rename(temp, filename);
    }
    return error;
}

//
// Load data from file.
//
Void RSSFeed::LoadFromFile(Void)
{
    SFCError error;
    SFXFile file;
    SFXAnsiStringStreamReader reader;
    SFXAnsiString string;
    SFXPath path(DIRECTORY_NAME + _filename);
    UInt32 size;

    // Get file size
    SFXFile::GetSize(path, &size);
    // Open file
    error = file.OpenReadOnly(path);
    // If the file was successfully opened
    if (error == SFERR_NO_ERROR) {
        // Get the reading stream
        error = file.GetStreamReader(size + 1, &reader);
        if (error == SFERR_NO_ERROR) {
            // Read from file
            reader.Fetch();
            // Save to string
            reader.ReadSFXAnsiString(&string);

            SInt32 c1 = 0;
            SInt32 c2, c3, c4, c5;
            while (true) {
                // Search for first line delimiter (end of line)
                c2 = string.FirstIndexOf(DELIMITER_STR, c1);
                // If not found, there is no line
                if (c2 < 0) {
                    break;
                }
                c3 = string.FirstIndexOf(DELIMITER_STR, c2 + 1);
                c4 = string.FirstIndexOf(DELIMITER_STR, c3 + 1);
                c5 = string.FirstIndexOf('\n', c4) + 1;
                SFXAnsiString dateNumber = string.Substring(c4 + DELIMITER_LEN, c5);
                SFXDate date;
                // Get the date in a number type
                date.Set(dateNumber.AsUInt32());
                // Get each attribute from string
                RSSItemPtr item = new RSSItem(string.Substring(c1, c2),
                string.Substring(c2 + DELIMITER_LEN, c3),
                date,
                string.Substring(c4 + DELIMITER_LEN, c5).Trim() == "1");
                // Append to the list
                if (_itemList.Append(item) != SFERR_NO_ERROR) {
                    ::delete item;
                    break;
                }
                c1 = c5;
            }
        }
        file.Close();
    }
    return;
}

//
// Look if the RSS item exists in the list. Return True if found.
//
Bool RSSFeed::Find(RSSItemConstRef item)
{
    // Get an iterator
    SFXList<RSSItemPtr>::Iterator itor(_itemList.GetFirstIterator());
    while(itor.HasNext()) {
        RSSItemPtr item2 = itor.GetNext();
        // RSS data exists
        if (item == *item2) {
            return true;
        }
    }
    return false;
}

//
// Desstructor definition.
//
RSSFeedList::~RSSFeedList(Void)
{
    RSSFeedPtr feed;
    // Get an iterator and release all data
    SFXArray<RSSFeedPtr>::Iterator itor = _feedArray.GetFirstIterator();
    while(itor.HasNext()) {
        feed = itor.GetNext();
        ::delete feed;
    }
    return;
}

//
// Add an RSS feed.
// Returns True if added successfully.
// A new file name for the feed itemList is set.
//
Bool RSSFeedList::Add(RSSFeedPtr feed)
{
    SFXAnsiString filename, temp;
    SFXLCGRandom rand;

    if (feed != null) {
        filename = feed->_title;
        filename += feed->_url;
        // The file name is prevented from overlapping using random numbers
        filename += SFXAnsiString::Format("%d", rand.GetUInt32() % 1000);
        // The file name is made random using MD5
        SFXMailUtility::GetMD5(filename, &temp, true);
        // The file name has 16 characters
        temp.SetLength(16);
        feed->SetFilename(temp);
        // Return True if added successfully
        return _feedArray.Append(feed) == SFERR_NO_ERROR;
    }
    return false;
}

//
// Save to file.
// The data specification is the following:
// - one RSS feed a line is shown,
// - each RSS item element is delimited with DELIMITER_STR,
// - when DELIMITER_STR is <>, a line becomes
// title<>description<>topic<>date<>alreadyRead\r\n
//
SFCError RSSFeedList::SaveToFile(Void)
{
    SFXPath temp;
    SFXPath filename(DIRECTORY_NAME LIST_FILE_NAME);
    SFXFile file;
    SFXAnsiStringStreamWriter writer;
    SFCError error;
    SInt32 i;

    // A temporary saving path is acquired
    error = SFXFile::GetTemporaryPath(DIRECTORY_PATH, &temp);

    if (error == SFERR_NO_ERROR) {
		bool exists;
        error =SFXFile::Exists(	temp ,&exists);
        if (exists==false)
        {
            file.Create(temp,true);
        }
        error = file.OpenReadWrite(temp);
        if (error == SFERR_NO_ERROR) {
            file.GetStreamWriter(4096, &writer);
            if (error == SFERR_NO_ERROR) {
                for (i = 0 ; i < _feedArray.GetSize() ; ++i) {
                    // Write to file
                    writer << _feedArray[i]->GetFilename() << DELIMITER_STR <<
                    _feedArray[i]->_title << DELIMITER_STR
                    << _feedArray[i]->_url << DELIMITER_STR
                    << SFXAnsiString::Format("%u",
                    _feedArray[i]->_date.AsUInt32())
                    << "\r\n";
                    writer.Flush();
                }
            }
            file.Close();
        }
    }
    if (error == SFERR_NO_ERROR) {
        SFXFile::Remove(filename);
        error = SFXFile::Rename(temp, filename);
    }
    return error;
}

//
// Load from file.
//
Void RSSFeedList::LoadFromFile(Void)
{
    SFCError error;
    SFXFile file;
    SFXAnsiStringStreamReader reader;
    SFXAnsiString string;
    SFXPath path(DIRECTORY_NAME LIST_FILE_NAME);
    UInt32 size;

    // Get the file size
    SFXFile::GetSize(path, &size);
    error = file.OpenReadOnly(path);
    // If file opening succeeded
    if (error == SFERR_NO_ERROR) {
        error = file.GetStreamReader(size + 1, &reader);
        if (error == SFERR_NO_ERROR) {
            reader.Fetch();
            // All data is read from the file
            reader.ReadSFXAnsiString(&string);

            SInt32 c1 = 0;
            SInt32 c2, c3, c4, c5;
            while (true) {
                // Search for the first line delimiter
                c2 = string.FirstIndexOf(DELIMITER_STR, c1);
                if (c2 < 0) {
                    break;
                }
                c3 = string.FirstIndexOf(DELIMITER_STR, c2 + 1);
                c4 = string.FirstIndexOf(DELIMITER_STR, c3 + 1);
                c5 = string.FirstIndexOf('\n', c4) + 1;
                SFXAnsiString dateNumber(
                    string.Substring(c4 + DELIMITER_LEN, c5).Trim());
                SFXDate date;
                date.Set(dateNumber.AsUInt32());
                RSSFeedPtr feed = new RSSFeed(
                    string.Substring(c2 + DELIMITER_LEN, c3),
                    string.Substring(c3 + DELIMITER_LEN, c4), date);
                // If added successfully
                if (Add(feed)) {
                    SFXAnsiString filename(string.Substring(c1, c2));
                    feed->SetFilename(filename);
                    feed->LoadFromFile();
                } else {
                    ::delete feed;
                    break;
                }
                c1 = c5;
            }
        }
        file.Close();
    }
    return;
}
